
 #include "FastLED.h"
 
 #define NUM_STRIP 6                 // количество лент
 #define NUM_LEDS 20                  // количество светодиодов в каждой ленте
 #define NUM_METEORS 12               // длина МЕТЕОРА
 #define INCREASE_BRIGHTNESS (256/NUM_METEORS)
 #define TIME_MIN 3                   // максимальная скорость метеора
 #define TIME_MAX 100                  // минимальная скорость метеора
 
 #define DATA_PIN1 2                  // выходы по АРДУИНО ProMicro для каждой ленты
 #define DATA_PIN2 3                  // подключать через резистор 47 Ом и желательно 10Ком на землю со стороны лент
 #define DATA_PIN3 4                  // питание лент от отдельного стабилизированного
 #define DATA_PIN4 5                 // источника постоянного напряжения 5V, не менее 5A
 #define DATA_PIN5 6                 // Земля общая с блоком питания Ардуино
 #define DATA_PIN6 7

 
 CRGB leds1[NUM_LEDS];                // первая лента
 CRGB leds2[NUM_LEDS];                // вторая лента
 CRGB leds3[NUM_LEDS];                // третья лента
 CRGB leds4[NUM_LEDS];                // четвертая лента
 CRGB leds5[NUM_LEDS];                // пятая лента
 CRGB leds6[NUM_LEDS];                // шестая лента
  
 CRGB meteors[NUM_METEORS];           // метеор
 CRGB etalone[NUM_METEORS+NUM_LEDS];  // эталонный массив метеор+лента
 CRGB rabochiy[NUM_METEORS+NUM_LEDS]; // рабочий массив для расчета лент
 byte LENTA[NUM_STRIP];               // массив признаков работы лент (1-выполняется метеор, 0-выкл)
 byte PHASE[NUM_STRIP];               // массив значений состояния сдвига для каждой ленты 0 - NUM_METEORS+NUM_LEDS
 byte PAUSE[NUM_STRIP];               // массив значений циклов задержки между метеорами для каждой ленты
 byte COLOR[NUM_STRIP];               // массив значений цвета лент
 
 int i,j,n = 0;
 int HSV, STAY_TIME;
 int t=80;                             // цвет и скорость метеора если число + то скорость -
 
void setup() { 
   
   // инициализируем ленты
   FastLED.addLeds<WS2812B, DATA_PIN1>(leds1, NUM_LEDS);
   FastLED.addLeds<WS2812B, DATA_PIN2>(leds2, NUM_LEDS);
   FastLED.addLeds<WS2812B, DATA_PIN3>(leds3, NUM_LEDS);
   FastLED.addLeds<WS2812B, DATA_PIN4>(leds4, NUM_LEDS);
   FastLED.addLeds<WS2812B, DATA_PIN5>(leds5, NUM_LEDS);
   FastLED.addLeds<WS2812B, DATA_PIN6>(leds6, NUM_LEDS);
   
   // назначение входов, настройка диапазона цветов метеора
   pinMode( A0, INPUT);  // Цвет
   pinMode( A1, INPUT); // Скорость
   pinMode( A2, INPUT); // Порог включения
   pinMode( A3, INPUT_PULLUP);  // Резерв под управление Stb, GPI
   HSV = GETCOLOR();
   STAY_TIME = t;
   
   // стартовая инициализация всех массивов
   for ( i = 0; i < NUM_STRIP; i++) {
   LENTA[i] = 1;
   PHASE[i] = 0;
   PAUSE[i] = 0;
   COLOR[i] = HSV;}
   
}
 
 
void loop() {
   for ( n = 0; n < NUM_STRIP; n++) {               // поочередно перебираем все ленты

   if ( LENTA[n] == 1) {                            // если на ленте есть МЕТЕОР то обрабатываем
   
     if ( PHASE[n] == (NUM_LEDS+NUM_METEORS)){      // если МЕТЕОР "вылетел" то
      PHASE[n] = 0;         // сброс ленты на НОЛЬ
      LENTA[n] = 0;                                 // выключение ленты  
        if (digitalRead(A3)==HIGH) {                // если тумблер выключен, то цвет согласно А0
          HSV = GETCOLOR();                         // подготовка нового цвета к следующему включению
          COLOR[n] = random(HSV-5,HSV+5);}          // и установка коридора флюктуации цвета
        else {COLOR[n]++;}                          // если тумблер включен, то плавно меняем цвет        
      PAUSE[n] = random(1,35); }                   // и задание паузы до следующего включения (в циклах)
      
   SHIFT(n,PHASE[n]);                               // если нормальный цикл обработки, то засвечиваем Метеор на ленте
   PHASE[n]++;}                                     // и сдвигаем фазу
   
   if (LENTA[n]==0) {                               // Если лента выключена
      if (PAUSE[n]>0) PAUSE[n]--;                   // то считаем длительность паузы
        if ((PAUSE[n]==0)&& (analogRead(A2) >=300)) LENTA[n]=1;}  // и по окончании включаем ленту при достаточной громкости (280-450),
              // в зависимости от чувствительности микрофона
  }

   FastLED.show();                                  // засветка всех лент
   STAY_TIME = map(analogRead(A1), 0, 50, TIME_MIN, TIME_MAX); // снятие показателя задержки (50-200)
   delay(STAY_TIME);                                // задержка по уровню скорости МЕТЕОРА
}



// Подпрограмма настройки  цвета по A0
int GETCOLOR()
   {
   int val = map(analogRead(A0), 3, 200, 0, 255);  // цветовой ряд HSV описывается в диапазоне от 0 до 255
   return val;                                      
   }                                                


// подпрограмма формирование метеора заданной длины и цвета и заливка его в эталонный массив
void METEOR(int color)
   {
   i,j = 0;
   for( i = INCREASE_BRIGHTNESS - 1; i < 256; i += INCREASE_BRIGHTNESS )     // создание МЕТЕОРА
   meteors[j++].setHSV( color, 255, i );                                     // заданного цвета
   for( i = 0; i < NUM_METEORS; i++)                                         // и сброс его в эталонный массив
   etalone[i] = meteors[i];
   for( i = NUM_METEORS; i < (NUM_LEDS + NUM_METEORS); i ++ )                // остальное заполняем "черным"
   etalone[i] = CRGB::Black;
   }
   

// подпрограмма копирования эталонного массива со сдвигом в рабочую область   
void SHIFT(int LED, int shift)
   {
   // подготовка своего цвета для каждой ленты
   if (LED==0) METEOR(COLOR[0]);        
   if (LED==1) METEOR(COLOR[1]);
   if (LED==2) METEOR(COLOR[2]);
   if (LED==3) METEOR(COLOR[3]);
   if (LED==4) METEOR(COLOR[4]);
   if (LED==5) METEOR(COLOR[5]);
    
   // переписываем эталон с указанным сдвигом в рабочий массив      
   for( i = (NUM_LEDS+NUM_METEORS-1); i > shift; i-- ) 
   rabochiy[i] = etalone[i-shift];
   for ( i = shift; i > 0; i--)
   rabochiy[i] = CRGB::Black;
   
   // и заливаем его на соответствующую ленту
   if (LED==0)   for( i = 0; i < NUM_LEDS; i++ ) leds1[i] = rabochiy[i+NUM_METEORS];  
   if (LED==1)   for( i = 0; i < NUM_LEDS; i++ ) leds2[i] = rabochiy[i+NUM_METEORS]; 
   if (LED==2)   for( i = 0; i < NUM_LEDS; i++ ) leds3[i] = rabochiy[i+NUM_METEORS]; 
   if (LED==3)   for( i = 0; i < NUM_LEDS; i++ ) leds4[i] = rabochiy[i+NUM_METEORS]; 
   if (LED==4)   for( i = 0; i < NUM_LEDS; i++ ) leds5[i] = rabochiy[i+NUM_METEORS]; 
   if (LED==5)   for( i = 0; i < NUM_LEDS; i++ ) leds6[i] = rabochiy[i+NUM_METEORS];
   }



